//=============================================================================
// Project:    SLP - An Interpreter for Super Logic Programs / Static Semantics
// Module:     rarg.h
// Purpose:    Argument of an atomic formula (literal) in a rule.
// Last Change:10.10.2001
// Language:   C++
// Author:     Stefan Brass
// Email:      Stefan.Brass@informatik.uni-giessen.de, sbrass@sis.pitt.edu
// Address:    Universitaet Giessen, Arndtstr. 2, 35392 Giessen, Germany
// Copyright:  (c) 2001 by Stefan Brass
// Licence:    Distributed under the conditions of the GNU Public Licence.
//=============================================================================

//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	RARG_INCLUDED
#define RARG_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef BOOL_INCLUDED
#include "bool.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif

#ifndef CNST_INCLUDED
#include "cnst.h"
#endif

#ifndef TERM_INCLUDED
#include "term.h"
#endif

#ifndef CNST_INCLUDED
#include "cnst.h"
#endif

#ifndef RVARS_INCLUDED
#include "rvars.h"
#endif

//=============================================================================
// Private Constants:
//=============================================================================

//-----------------------------------------------------------------------------
// RARG_MAGIC: Magic number (identifies objects of this class for debugging).
//-----------------------------------------------------------------------------

#ifdef VER_DEBUG
static const long RARG_MAGIC = 0x52415200L;	// 'RAR\0'
#endif

//=============================================================================
// Class for Terms in Atomic Formulas (Literals) in Rules.
//=============================================================================

class rarg_c {

	public:

//-----------------------------------------------------------------------------
// Methods:
//-----------------------------------------------------------------------------

	// Constructor (transform term from formula into rule):
	rarg_c(term_t term, rvars_t vars, bool_t binding);

	// Destructor:
	~rarg_c(void);

	// Is this argument term a free variable (not yet bound to a value)?
	inline bool_t is_free(void) const
		{ CHECK(valid(), "rarg_c::is_free: Bad object");
		  return(Kind == RARG_FREE); }

	// Is this argument term a bound variable (bound to a value)?
	inline bool_t is_bound(void) const
		{ CHECK(valid(), "rarg_c::is_bound: Bad object");
		  return(Kind == RARG_BOUND); }

	// Is this argument term the anonymus variable?
	inline bool_t is_anon(void) const
		{ CHECK(valid(), "rarg_c::is_anon: Bad object");
		  return(Kind == RARG_ANON); }

	// Is this argument term a ground term (atom or integer)?
	inline bool_t is_cnst(void) const
		{ CHECK(valid(), "rarg_c::is_cnst: Bad object");
		  return(Kind == RARG_CNST); }

	// Return the offset of the variable of this argument term:
	inline int get_offset(void) const
		{ CHECK(valid(), "rarg_c::get_offset: Bad object");
		  CHECK(Kind == RARG_FREE || Kind == RARG_BOUND,
			"rarg_c::get_offset: term isn't variable");
		  return(Data.Offset); }

	// Return the constant of this term:
	inline cnst_t get_cnst(void) const
		{ CHECK(valid(), "rarg_c::get_cnst: Bad object");
		  CHECK(Kind == RARG_CNST,
			"rarg_c::get_const: term isn't constant");
		  return(Data.Cnst); }

	// Print this Term:
	void	print(rvars_t vars) const;

//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
	// Integrity Check:
	public:
	bool_t	valid(void) const;

	// Magic number (identifies objects of this class for debugging).
	private:
	long	Magic;	// Must be "RARG_MAGIC"
#endif

//-----------------------------------------------------------------------------
// Private Members:
//-----------------------------------------------------------------------------

	private:

	// Kind: Which kind of term is this?
	enum {RARG_FREE, RARG_BOUND, RARG_ANON, RARG_CNST} Kind;

	// Data: The offset of the variable, or pointer to constant (atom/int).
	union {int Offset; cnst_t Cnst;} Data;

//-----------------------------------------------------------------------------
// End of Class:
//-----------------------------------------------------------------------------

};

//-----------------------------------------------------------------------------
// Define pointer type:
//-----------------------------------------------------------------------------

typedef rarg_c *rarg_t;

//=============================================================================
// End of Include File:
//=============================================================================

#endif

